"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const _ = require("underscore");
const utils_1 = require("./utils");
const declarations_1 = require("./declarations");
const params_1 = require("./params");
const overrides_1 = require("./overrides");
function Config(options) {
    return (cls) => {
        const meta = Reflect.getMetadata(declarations_1.METADATA_KEY, cls) || {
            middlewares: [],
            handlers: {},
            routerOptions: {},
        };
        _.extend(meta.routerOptions, options);
        Reflect.defineMetadata(declarations_1.METADATA_KEY, meta, cls);
    };
}
exports.Config = Config;
function Handler(options = {}) {
    return (target, propertyKey) => {
        const cls = target.constructor;
        const meta = Reflect.getMetadata(declarations_1.METADATA_KEY, cls) || {
            middlewares: [],
            handlers: {},
            routerOptions: {},
        };
        if (meta.handlers[propertyKey] == null) {
            meta.handlers[propertyKey] = {
                method: declarations_1.Method.GET,
                path: '/',
                middleware: null,
            };
        }
        if (options.method) {
            meta.handlers[propertyKey].method = options.method;
        }
        if (options.path) {
            meta.handlers[propertyKey].path = options.path;
        }
        if (meta.handlers[propertyKey].path == null) {
            meta.handlers[propertyKey].path = '/';
        }
        Reflect.defineMetadata(declarations_1.METADATA_KEY, meta, cls);
    };
}
exports.Handler = Handler;
function Get(path, options = {}) {
    return Handler(_.extend({}, options, {
        path,
        method: declarations_1.Method.GET,
    }));
}
exports.Get = Get;
function Post(path, options = {}) {
    return Handler(_.extend({}, options, {
        path,
        method: declarations_1.Method.POST,
    }));
}
exports.Post = Post;
function Put(path, options = {}) {
    return Handler(_.extend({}, options, {
        path,
        method: declarations_1.Method.PUT,
    }));
}
exports.Put = Put;
function Delete(path, options = {}) {
    return Handler(_.extend({}, options, {
        path,
        method: declarations_1.Method.DELETE,
    }));
}
exports.Delete = Delete;
function Middleware(middlewares) {
    middlewares = _.flatten([middlewares]);
    return (target, propertyKey, descriptor) => {
        if (propertyKey == null) {
            buildConstructorMiddleware(target, middlewares);
        }
        else {
            buildPropertyMiddleware(target, propertyKey, middlewares, descriptor);
        }
    };
}
exports.Middleware = Middleware;
function buildConstructorMiddleware(cls, middlewares) {
    const meta = _.clone(Reflect.getMetadata(declarations_1.METADATA_KEY, cls) || {
        middlewares: [],
        handlers: {},
        routerOptions: {},
    });
    meta.middlewares = _.union(middlewares, meta.middlewares);
    Reflect.defineMetadata(declarations_1.METADATA_KEY, meta, cls);
}
function buildPropertyMiddleware(target, propertyKey, middlewares, descriptor) {
    const cls = target.constructor;
    const meta = _.clone(Reflect.getMetadata(declarations_1.METADATA_KEY, cls) || {
        middlewares: [],
        handlers: {},
        routerOptions: {},
    });
    if (meta.handlers[propertyKey] == null) {
        meta.handlers[propertyKey] = {
            method: declarations_1.Method.GET,
            path: null,
            middleware: null,
        };
    }
    if (descriptor != null) {
        descriptor.value = utils_1.compose(middlewares.concat(descriptor.value));
    }
    else {
        const m = meta.handlers[propertyKey];
        if (m.middleware == null) {
            m.middleware = utils_1.compose(middlewares);
        }
        else {
            m.middleware = utils_1.compose(middlewares.concat(m.middleware));
        }
    }
    Reflect.defineMetadata(declarations_1.METADATA_KEY, meta, cls);
}
exports.Overrides = (...rules) => {
    return Middleware(overrides_1.overrides(...rules));
};
exports.Params = (options) => {
    return Middleware(params_1.params(options));
};

//# sourceMappingURL=decorators.js.map
